@@ -1,6 +1,6 @@
 ;;; dgnushack.el --- a hack to set the load path for byte-compiling
 ;; Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2003,
-;; 2004, 2005
+;; 2004, 2005, 2006, 2007
 ;;        Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
@@ -118,12 +118,15 @@ than subr.el."
 	ad-do-it))))
 
 (when (and (not (featurep 'xemacs))
-	   (byte-optimize-form '(and (> 0 1) foo) t))
+	   (byte-optimize-form
+	    '(and (> 0 1)
+		  (message "This should not appear in the byte-code."))
+	    t))
   (defadvice byte-optimize-form-code-walker
     (around fix-bug-in-and/or-forms (form for-effect) activate)
     "Optimize the rest of the and/or forms.
 It has been fixed in XEmacs before releasing 21.4 and also has been
-fixed in Emacs after 21.3."
+fixed in Emacs 22."
     (if (and for-effect (memq (car-safe form) '(and or)))
 	(let ((fn (car form))
 	      (backwards (reverse (cdr form))))
@@ -151,6 +154,11 @@ fixed in Emacs after 21.3."
 ;;
 ;; http://thread.gmane.org/gmane.emacs.xemacs.beta/20519
 ;; Subject: XEmacs 21.5 and Gnus fancy splitting.
+;;
+;; Should be fixed in XEmacs (March 2007).
+;; http://thread.gmane.org/gmane.emacs.xemacs.patches/8124
+;; When should we remove this workaround?
+;;
 (when (and (featurep 'xemacs)
 	   (let ((table (copy-syntax-table emacs-lisp-mode-syntax-table)))
 	     (modify-syntax-entry ?= " " table)
@@ -201,6 +209,7 @@ fixed in Emacs after 21.3."
     (autoload 'apropos-command "apropos" nil t)
     (autoload 'bbdb-complete-name "bbdb-com" nil t)
     (autoload 'browse-url "browse-url" nil t)
+    (autoload 'browse-url-of-file "browse-url" nil t)
     (autoload 'c-mode "cc-mode" nil t)
     (autoload 'customize-apropos "cus-edit" nil t)
     (autoload 'customize-group "cus-edit" nil t)
@@ -232,7 +241,9 @@ fixed in Emacs after 21.3."
     (autoload 'smtpmail-send-it "smtpmail")
     (autoload 'sort-numeric-fields "sort" nil t)
     (autoload 'sort-subr "sort")
+    (autoload 'toggle-truncate-lines "view-less" nil t)
     (autoload 'trace-function-background "trace" nil t)
+    (autoload 'unmorse-region "morse" nil t)
     (autoload 'w3-do-setup "w3")
     (autoload 'w3-prepare-buffer "w3-display")
     (autoload 'w3-region "w3-display" nil t)
