@@ -1,7 +1,7 @@
 ;;; gnus-draft.el --- draft message support for Gnus
 
 ;; Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-;;   2005, 2006 Free Software Foundation, Inc.
+;;   2005, 2006, 2007 Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
 ;; Keywords: news
@@ -76,7 +76,6 @@
       (when (gnus-visual-p 'draft-menu 'menu)
 	(gnus-draft-make-menu-bar))
       (add-minor-mode 'gnus-draft-mode " Draft" gnus-draft-mode-map)
-      (mml-mode)
       (gnus-run-hooks 'gnus-draft-mode-hook))))
 
 ;;; Commands
@@ -106,7 +105,9 @@
       (save-restriction
 	(message-narrow-to-headers)
 	(message-remove-header "date")))
-    (save-buffer)
+    (let ((message-draft-headers
+	   (delq 'Date (copy-sequence message-draft-headers))))
+      (save-buffer))
     (let ((gnus-verbose-backends nil))
       (gnus-request-expire-articles (list article) group t))
     (push
@@ -239,6 +240,12 @@
 		    (throw 'continue t)
 		  (error "Stop!"))))))))
 
+(defcustom gnus-draft-setup-hook nil
+  "Hook run after setting up a draft buffer."
+  :group 'gnus-message
+  :version "23.0" ;; No Gnus
+  :type 'hook)
+
 ;;; Utility functions
 
 ;;;!!!If this is byte-compiled, it fails miserably.
@@ -286,7 +293,8 @@
 		(gnus-add-mark ,(car ga) 'replied ,article)
 		(gnus-request-set-mark ,(car ga) (list (list (list ,article)
 							     'add '(reply)))))
-	     'send)))))))
+	     'send))))
+      (run-hooks 'gnus-draft-setup-hook))))
 
 (defun gnus-draft-article-sendable-p (article)
   "Say whether ARTICLE is sendable."
