@@ -10,54 +10,58 @@
   (mapcar (lambda (var) (unless (boundp var) (set var nil))) args))
 
 (maybe-fbind '(Info-directory
-	       Info-menu create-image display-graphic-p
-	       display-time-event-handler find-coding-system find-image
-	       image-size image-type-available-p insert-image
-	       line-number-at-pos make-mode-line-mouse-map make-temp-file
-	       propertize put-image replace-regexp-in-string
-	       rmail-msg-is-pruned rmail-msg-restore-non-pruned-header
-	       sort-coding-systems tool-bar-add-item
-	       tool-bar-add-item-from-menu tool-bar-local-item-from-menu
+	       Info-menu clear-string cp-supported-codepages create-image
+	       custom-autoload display-graphic-p display-time-event-handler
+	       epg-check-configuration find-coding-system find-image image-size
+	       image-type-available-p insert-image line-number-at-pos
+	       make-mode-line-mouse-map make-network-process make-temp-file
+	       propertize put-image rmail-msg-is-pruned
+	       rmail-msg-restore-non-pruned-header sort-coding-systems
+	       tool-bar-add-item tool-bar-add-item-from-menu
+	       tool-bar-local-item-from-menu unicode-precedence-list
 	       url-generic-parse-url url-http-file-exists-p
 	       url-insert-file-contents vcard-pretty-print w32-focus-frame
 	       w3m-detect-meta-charset w3m-region x-focus-frame))
 (maybe-bind '(eudc-protocol
 	      filladapt-mode rmail-insert-mime-forwarded-message-function
-	      w3-meta-charset-content-type-regexp
+	      url-version w3-meta-charset-content-type-regexp
 	      w3-meta-content-type-charset-regexp))
 
 (if (featurep 'xemacs)
     (progn
       (maybe-fbind '(codepage-setup
-		     delete-overlay detect-coding-string eudc-expand-inline
-		     event-click-count event-end event-start
+		     delete-overlay detect-coding-string display-images-p
+		     eudc-expand-inline event-click-count event-end event-start
 		     find-coding-systems-for-charsets
 		     find-coding-systems-region find-coding-systems-string
 		     mail-abbrevs-setup mouse-minibuffer-check mouse-movement-p
 		     mouse-scroll-subr overlay-lists posn-point posn-window
-		     read-event set-buffer-multibyte track-mouse window-edges))
+		     read-event set-buffer-multibyte
+		     select-frame-set-input-focus select-safe-coding-system
+		     string-as-multibyte timer-set-function track-mouse
+		     window-edges))
       (maybe-bind '(adaptive-fill-first-line-regexp
 		    buffer-display-table buffer-file-coding-system
-		    current-language-environment
+		    current-language-environment cursor-in-non-selected-windows
 		    default-enable-multibyte-characters
 		    enable-multibyte-characters gnus-agent-expire-current-dirs
 		    language-info-alist mark-active mouse-selection-click-count
 		    mouse-selection-click-count-buffer pgg-parse-crc24
-		    temporary-file-directory transient-mark-mode)))
+		    temporary-file-directory timer-list transient-mark-mode)))
   (maybe-fbind '(bbdb-complete-name
 		 delete-annotation delete-extent device-connection dfw-device
-		 events-to-keys font-lock-set-defaults frame-device
+		 events-to-keys find-face font-lock-set-defaults frame-device
 		 get-char-table glyph-height glyph-width ldap-search-entries
 		 mail-aliases-setup make-annotation make-event make-glyph
-		 make-network-process map-extents message-xmas-redefine
-		 put-char-table set-extent-property temp-directory
+		 map-extents message-xmas-redefine put-char-table
+		 set-extent-property set-itimer-function temp-directory
 		 valid-image-instantiator-format-p
 		 w3-coding-system-for-mime-charset w3-do-setup
 		 w3-prepare-buffer w3-region window-pixel-height
 		 window-pixel-width))
-  (maybe-bind '(help-echo-owns-message mail-mode-hook mm-w3m-mode-map
-				       show-nonbreak-escape
-				       nobreak-char-display)))
+  (maybe-bind '(help-echo-owns-message
+		itimer-list mail-mode-hook mm-w3m-mode-map nobreak-char-display
+		show-nonbreak-escape)))
 
 (when (and (featurep 'xemacs)
 	   (not (featurep 'mule)))
@@ -67,9 +71,10 @@
 		 pgg-parse-crc24-string))
   (unless (featurep 'file-coding)
     (maybe-fbind '(coding-system-base
-		   coding-system-list coding-system-p decode-coding-region
-		   decode-coding-string detect-coding-region
-		   encode-coding-region encode-coding-string))))
+		   coding-system-change-eol-conversion coding-system-list
+		   coding-system-p decode-coding-region decode-coding-string
+		   detect-coding-region encode-coding-region
+		   encode-coding-string))))
 
 (defun nnkiboze-score-file (a)
   )
